<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Teknisa\TCO\Services\Tco00400TipoContratoService;

class Tco00300ServicoProposta extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }

    public function salvaNovoServicoProposta(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $servicoPropostaExistente = $this->retornaServicoPropostaExistente($row);
        if(empty($servicoPropostaExistente)){
            if($this->isValid($row, $response)){
                $row["CDSERVICOPROPOSTA"] = str_pad($row["CDSERVICOPROPOSTA"], 5, "0", STR_PAD_LEFT);
                $this->salvaServicoProposta($request);
                $response->addDataSet(new DataSet("servicoProposta", array()));
                $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
            }
        } else {
            $response->addDataSet(new DataSet("servicoProposta", $servicoPropostaExistente));
        }
    }
    
    private function isValid($row, $response) {
        $arrayErrors = array();
        
        if(strlen($row["CDSERVICOPROPOSTA"]) > 5){
            $response->addMessage(new Message('TAMANHO_CODIGO'));
            return false;
        }
        if(preg_match('/^([0-9]*)$/', $row["CDSERVICOPROPOSTA"]) < 1){
            $response->addMessage(new Message('CODIGO_INVALIDO'));
            return false;
        }
        if(strlen(iconv("UTF-8", "ISO-8859-1", $row["NMSERVICOPROPOSTA"])) > 50){
            $response->addMessage(new Message('TAMANHO_NOME'));
            return false;
        }
        if(strlen($row["NMSERVICOPROPOSTA"]) == 0){
            $response->addMessage(new Message('NOME_VAZIO'));
            return false;
        }
        
        $paramTCOFilial = $this->retornaParamTCOFilial();
        if(isset($row["CDFILIAL"]) && $row["CDFILIAL"] != $paramTCOFilial["CDFILIAL"]){
            $response->addMessage(new Message('TCO00300_SERVICO_PROPOSTA_PARAMTCO_FILIAL'));
            return false;
        }
        
        return true;
    }
    
    private function retornaParamTCOFilial() {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT CDFILIAL
                  FROM PARAMTCO";
        
        $query = $connection->prepare($sql);
        
        $query->execute(array());
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function retornaServicoPropostaExistente($row) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT 'TCO00300_SERVICO_PROPOSTA_EXISTENTE' AS MESSAGECODE,
                       NRSERVICOPROPOSTA,
                       CDSERVICOPROPOSTA,
                       NMSERVICOPROPOSTA
                  FROM SERVICOPROPOSTA
                 WHERE CDSERVICOPROPOSTA = :CDSERVICOPROPOSTA
                   AND (NRSERVICOPROPOSTA <> :NRSERVICOPROPOSTA OR :NRSERVICOPROPOSTA IS NULL)
                   AND NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "CDSERVICOPROPOSTA"  => str_pad($row["CDSERVICOPROPOSTA"], 5, "0", STR_PAD_LEFT),
            "NRSERVICOPROPOSTA"  => isset($row['NRSERVICOPROPOSTA']) ? $row['NRSERVICOPROPOSTA'] : null,
            "NRORG"              => $this->environment->getNrOrgTrab()
        ); 
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function salvaServicoProposta($request) {
        $nomenovoservico = $request->getRow()["NMSERVICO"];
        $rows = $this->crudService->getRowsToSave($request, "servicoproposta");
        $this->crudService->save("servicoproposta", $rows);
        $rows[0]["__is_new"] = false;
        $rows[0]["NMSERVICO"] = $nomenovoservico;
        $this->crudService->addColumnToReturn($rows[0]);
    }
    
    public function removeServicoPropostaSelecionado(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $servicoPropostaEmUso = $this->retornaServicoPropostaEmUso($row);
        
        if(empty($servicoPropostaEmUso)){
            $this->removeServicoProposta($request, $response);
            $response->addDataSet(new DataSet("servicoPropostaEmUso", array()));
        } else {
            $response->addDataSet(new DataSet("servicoPropostaEmUso", $servicoPropostaEmUso));
        }
        
    }
    
    private function retornaServicoPropostaEmUso($row) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT MESSAGECODE,
                       PARAM1,
                       PARAM2,
                       PARAM3
                 FROM (  /*Serviço para Proposta em uso no cardápio base __PARAM1__*/
                         SELECT 'TCO00300_SERVICO_PROPOSTA_EM_USO_CARDAPIOBASE' AS MESSAGECODE,                
                                C.CDCARDAPIOBASE AS PARAM1,
                                NULL AS PARAM2,
                                NULL AS PARAM3
                           FROM CARDAPIOBASE C
                         WHERE C.NRSERVICOPROPOSTA = :NRSERVICOPROPOSTA
                          UNION ALL
                         /*Serviço para Proposta em uso na proposta comercial __PARAM1__ simulação __PARAM2__*/
                         SELECT 'TCO00300_SERVICO_PROPOSTA_EM_USO_SERVICOPROPCOM' AS MESSAGECODE,                
                                P.CDPROPOSTACOMERCIAL AS PARAM1,                
                                V.CDVERSAOPROPCOM AS PARAM2,
                                NULL AS PARAM3
                           FROM SERVICOPROPCOM S
                                INNER JOIN SERVICOPROPOSTA SP ON SP.NRSERVICOPROPOSTA = S.NRSERVICOPROPOSTA
                                INNER JOIN VERSAOPROPCOM V ON V.NRVERSAOPROPCOM = S.NRVERSAOPROPCOM
                                INNER JOIN PROPOSTACOMERCIAL P ON P.NRPROPOSTACOMERCIAL =  V.NRPROPOSTACOMERCIAL
                         WHERE S.NRSERVICOPROPOSTA = :NRSERVICOPROPOSTA
                         
                         /*UNION ALL
                         /*Serviço para Proposta em uso no serviço __PARAM1__ da unidade __PARAM2__
                         SELECT 'TCO00300_SERVICO_PROPOSTA_EM_USO_SERVICO' AS MESSAGECODE,                
                                S.CDSERVFILI || ' - ' || S.NMSERVICO AS PARAM1,
                                S.CDFILIAL AS PARAM2,
                                NULL AS PARAM3
                           FROM SERVICO S
                          WHERE S.NRSERVICOPROPOSTA = :NRSERVICOPROPOSTA*/
                         
                     )";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRSERVICOPROPOSTA"  => $row['NRSERVICOPROPOSTA']
        ); 
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function removeServicoProposta($request, $response) {
        $rows = $this->crudService->getRowsToDelete($request, "servicoproposta");
        $dataSet = new DataSet("servicoproposta", $rows);
		$this->crudService->delete($response, $dataSet);
    }
    
}
